import io
import zipfile

from .qr import create_qr_code


def generate_qr_zip(codes, form):
    """

    :type codes: list[qrtoolkit_core.models.QRCode]
    :type form: qrtoolkit_core.generator.forms.QrGenerateForm
    """
    s = io.BytesIO()

    zf = zipfile.ZipFile(s, mode='w', compression=zipfile.ZIP_DEFLATED)

    for code in codes:
        code_buffer = create_qr_code(code, form)
        filename = form.create_filename(code)
        zf.writestr(filename, code_buffer)

    zf.close()

    return s.getvalue()



import io
import segno
from django.conf import settings


def create_qr_code(code, form):
    """

    :type code: qrtoolkit_core.models.QRCode
    :type form: qrtoolkit_core.generator.forms.QrGenerateForm
    """
    out = io.BytesIO()
    url = f'{settings.REDIRECT_SERVICE_URL}/{code.short_uuid}/'
    qr = segno.make_qr(url)
    qr.save(out,
            dark=form.cleaned_data['dark'],
            light=form.cleaned_data['light'],
            scale=form.cleaned_data['scale'],
            kind=form.cleaned_data['kind'])
    return out.getvalue()



from django.core.exceptions import ValidationError


def validate_is_api_mode_qrcode(value):
    """

    :type value: LinkUrl
    """
    from .models import QRCode, LinkUrl
    qrcode = LinkUrl.objects.select_related('code').get(pk=value).code
    if qrcode.mode != QRCode.REDIRECT_MODE_CHOICES.API_CALL:
        raise ValidationError('The selected qr code is not in Api call Mode.')



# Generated by Django 3.0.11 on 2021-06-30 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0007_auto_20210630_1417'),
    ]

    operations = [
        migrations.AlterField(
            model_name='apihit',
            name='action',
            field=models.CharField(choices=[('basic_info', 'Basic Info'), ('kiosk', 'Kiosk'), ('json', 'Json Response'), ('redirect', 'Redirect'), ('api_call', 'Api Call'), ('error', 'Error')], default='basic_info', max_length=16),
        ),
    ]



from django.conf import settings
from django.db import migrations, models


def remove_error_hits(apps, schema_editor):
    ApiHit = apps.get_model("qrtoolkit_core", "ApiHit")
    db_alias = schema_editor.connection.alias
    ApiHit.objects.using(db_alias).filter(message__icontains='does not exist.').delete()


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('qrtoolkit_core', '0004_auto_20210512_0733'),
    ]

    operations = [
        migrations.RunPython(remove_error_hits)
    ]



from django.urls import path
from django.urls.conf import include
from rest_framework.urlpatterns import format_suffix_patterns
from rest_framework import routers
from rest_framework.schemas import get_schema_view

from . import views
from . import viewsets

router = routers.DefaultRouter()
router.register(r'qrcodes', viewsets.CodeViewSet, basename='api-code')
router.register(r'apihits', viewsets.ApiHitViewSet, basename='api-apihit')
router.register(r'departments', viewsets.DepartmentViewSet, basename='api-department')
router.register(r'urls', viewsets.LinkUrlViewSet, basename='api-url')

api_routes = [path('', include(router.urls)),
              path('openapi/', get_schema_view(
                  title="Qr code Toolkit API",
                  description="Stad Gent qr code toolkit",
                  version="1.0.0",
                  patterns=router.urls,
                  url='/api/'
              ), name='openapi-schema')
              ]


def _get_code_routes():
    redirect_routes = [
        # url that ill be scanned
        path('<slug:short_uuid>/', views.QRCodeDetails.as_view(), name='qrcode-detail')
    ]
    # enable content negotiation on scan url
    redirect_routes = format_suffix_patterns(redirect_routes, allowed=['html', 'json'])
    info_routes = [
        path('code/<slug:short_uuid>/', views.CodeView.as_view(), name='code-detail'),
    ]
    link_url_routes = [
        path('call/<int:pk>/', views.link_url_clicked, name='link_url_clicked')
    ]
    return redirect_routes + info_routes + link_url_routes


code_routes = _get_code_routes()
